#!/usr/bin/perl -Pw

# ********** sigd - a SIGnature Daemon
# * Copyright (C) 2001,2002 by Gerfried Fuchs <http://alfie.ist.org/alfie/>
# *               see the end of the file for more details
# *****************************************************************************
# *
#define DATE      "2002-10-05"
#define VERSION   "0.10.0"
# *
# * Purpose: This program takes the files of the directory specified with the
# *          -d=<dir> or -dir=<dir> option (default: "~/.sig") and writes them
# *          randomly to the fifo specified with the -f=<file> or -file=<file>
# *          option (default: "~/.signature"). This way any signature that is
# *          requested is a random file from within the directory. The files
# *          MUST NOT start with a dot (or they would be ignored ;)
# *          You could also use this daemon for your .plan-file or something
# *          similar...
# * Usage:   sigd [OPTION]...
# *          Where OPTION is one or more of the following:
# *          -h  --help          prints out a usage message
# *          -f  --file=<file>   speicfy the file to use for the pipe
# *          -d  --dir=<dir>     specify directory to get the files from
# *          -l  --line=<text>   specifies a line to be put atop of all files
# *          -r  --rnd           files will be used randomly [default]
# *          -s  --seq           files will be used one after the other
# *          -v  --version       prints out the version of the program
# *          -c  --copy          print out the copyright notice
# *****************************************************************************

use strict;    # every single perl program needs it
use Getopt::Mixed qw(nextOption);       # for option handling
use POSIX qw(setsid mkfifo EPERM);      # daemonize, pipe

## default settings
#define FILENAME  ".signature"
#define SIGDIR    ".sig"
## please notice, your umasks influences the permissions
#define PERMS     0644
#define LOCKEXT   ".pid"

my ($cmnd) = $0 =~ /([^\/]+)$/;




######################
### OPTION PARSING ###
######################

my $sigpipe     = "$ENV{'HOME'}/".FILENAME;
my $sigdir      = "$ENV{'HOME'}/".SIGDIR;
my $sigd_pidext = LOCKEXT;
my ($line, $rnd, $seq);

Getopt::Mixed::init( # option handling {{{
    'h f=s d=s l=s r s v c' .
    ' help>h file>f dir>d line>l rnd>r seq>s version>v copy>c');

while (my ($option, $value, $pretty) = nextOption()) {
    if      ($option eq 'h') {   # print out the help
            print "Usage: $cmnd [OPTION]...
Following options are available:
  -h  --help          prints out this help page
  -f  --file=<file>   specifies the file to use for the pipe
  -d  --dir=<dir>     specifies the directory to read the files from
  -l  --line=<line>   specifies a line to be put atop of all the files
  -r  --rnd           files will be used randomly [default]
  -s  --seq           files will be used one after the other (sequentiell)
  -v  --version       prints out the version of the program
  -c  --copy          prints out the copyright notice
";
        exit;

    } elsif ($option eq 'f') {   # filename for the pipe
        $sigpipe = $value;

    } elsif ($option eq 'd') {   # directory with the signatures
        $sigdir = $value;

    } elsif ($option eq 'l') {   # line to prepend to files
        $line = $value;

    } elsif ($option eq 'r') {   # random order (default)
        $rnd = 'random';

    } elsif ($option eq 's') {   # sequential order
        $seq = 'sequential';

    } elsif ($option eq 'v') {   # print out the version number
        print ("$cmnd ".VERSION.' ('.DATE.")\n");
        exit;

    } elsif ($option eq 'c') {   # print out the copyright notice
        print (<DATA>);
        exit;
    }
}
Getopt::Mixed::cleanup(); # }}}
die "$cmnd: can't use both -r and -s\n" if ($rnd && $seq);

my $pidfile = $sigpipe.$sigd_pidext;




####################
### MAIN PROGRAM ###
####################

if (-f $pidfile) { # Locking: allow just one running sigd per file {{{
    open(PID, "<$pidfile") || die "$cmnd: can't read '$pidfile': $!\n";
    my $pid = <PID>;
    close PID;
    print "pid: $pid\n";

    if ($pid ne '') {       # does process exist?
        if (kill(0, $pid)) {
            die "$cmnd: already running (PID=$pid)\n";
        }
    }
}
unlink $sigpipe;    # make space for the sigpipe
unlink $pidfile;    # rm stale lockfile
# write the pid *after* putting ourself in daemon mode.... }}}

srand($$|time());    # initialize random number
my @file;

$SIG{'TERM'}= 'sig_handler';     # set the different sig_handlers
$SIG{'INT'} = 'sig_handler';
$SIG{'HUP'} = 'load_sigs';
$SIG{'PIPE'}= 'sig_handler';

&load_sigs();

&daemonize;
# correct place to write the new pid
open(PID, ">$pidfile") || die "$cmnd: can't write '$pidfile': $!\n";
print PID "$$";
close PID;

my $i = 0;
while (1) {
    mkfifo($sigpipe, PERMS) || die "$cmnd: can't create $sigpipe: $!\n";

    if (defined($seq)) {
        $i==$#file ? $i=0 : $i++;
    } else {
        $i = int rand ($#file+1);
    }

    while (1) {
        last if (open (R, "<$sigdir/$file[$i]"));
        &load_sigs;
    }

    open (W, ">$sigpipe") || die "$cmnd: can't open $sigpipe: $!\n";
    print W ($line."\n") if $line;
    print W (<R>);
    close (R);
    close (W);

    (unlink $sigpipe) || die "$cmnd: can't unlink $sigpipe: $!\n";
}




####################
### SUB ROUTINES ###
####################

### SIG HANDLER ###        exit program when signal occures {{{
###################
sub sig_handler {
    my ($signal) = @_;   # signal which interupted
    unlink $sigpipe;
    unlink $pidfile;
    print STDERR ("$cmnd: killed by $signal\n");
    exit;
} # }}}

### LOAD SIGNATURES ###    (re)load the signature list {{{
#######################
sub load_sigs {
    my $sigs = 0;
    @file = ();
    opendir (DIR, $sigdir) || die "$cmnd: can't read $sigdir: $!\n";
    while (my $entry = readdir(DIR)) {     # get the signature names
       $file[$sigs++] = $entry if (substr($entry,0,1) ne '.');
    }
    closedir (DIR);
    die "$cmnd: no signatures found!" if ($#file < 0);
} # }}}

### DAEMONIZE ###          put ourself in daemon mode {{{
#################
sub daemonize {
    open STDIN, '/dev/null' or die "$cmnd: can't read /dev/null: $!\n";
    open STDOUT, '>/dev/null';
    open STDERR, '>/dev/null';
    die "$cmnd: can't fork: $!\n" unless defined (my $rc = fork);
    exit if $rc;
    umask 0022;
    setsid or die "$cmnd: can't start new session: $!\n";
} # }}}




__END__
********** sigd - a SIGnature Daemon
* Copyright (C) 2001,2002 by Gerfried Fuchs <http://alfie.ist.org/alfie/>
* 
* Maintainer:  Gerfried Fuchs <alfie@ist.org>
* Homepage:    <http://alfie.ist.org/projects/sigd/>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*****************************************************************************
